clear;
close all;
clc;

% STEP 1: copy your data (streamflow q and drainage density dd) in the attached excel file

% STEP 2: specify the value of dpot here, or use NaN to have it calibrated.
opts.dpot = 3.68; % [km-1] potential drainage density

% STEP 3: specify here the initial values of the parameters to calibrate
opts.startPoint = []; % This is useful only if the autometic fitting does not give good results. Possiblilties:
					  % []				to use autometic values
					  % [dpot k theta]  if dpot is being calibrated
					  % [k theta]		if dpot is fixed

% STEP 4: run code!

% analysis
data = importData();
opts = createOptions(opts);
model = fitCurve(data.q, data.dd, opts);

% show results
plotResult(data, model);

disp(model);
disp(model.parameters);

%% FUNCTIONS
function excel = importData()
	opts = spreadsheetImportOptions("NumVariables", 6);

	% Specify sheet and range
	opts.Sheet = 1;
	opts.DataRange = [5 104];

	% Specify column names and types
	opts.VariableNames = ["Var1", "Var2", "Var3", "q", "dd"];
	opts.SelectedVariableNames = ["q", "dd"];
	opts.VariableTypes = ["char", "char", "char", "double", "double"];

	% Specify variable properties
	opts = setvaropts(opts, ["Var1", "Var2", "Var3"], "WhitespaceRule", "preserve");
	opts = setvaropts(opts, ["Var1", "Var2", "Var3"], "EmptyFieldRule", "auto");

	% Import the data
	excel = readtable("excel.xlsx", opts, "UseExcel", false);

	% Remove NaN
	id = isnan(excel.q) | isnan(excel.dd);
	excel = excel(~id, :);
end

function opts = createOptions(opts)
	if isnan(opts.dpot)
		opts.parName = ["dpot", "k", "theta"];
		opts.fitType = fittype(@(dpot, k, theta, x) dpot * gammainc(x/theta, k));
		opts.lowerBound = @(dd, q) [0 0 0];
		opts.problemParam = {};
		opts.label = "Gamma model (calibrated ddpot)";

		if isempty(opts.startPoint)
			opts.startPoint = @(dd, q) [1.2*max(dd) 10 1];
		else
			stp = opts.startPoint;
			opts.startPoint = @(dd, q) stp; 
		end
	else
		opts.parName = ["k", "theta"];
		opts.fitType = fittype(@(k, theta, dpot, x) dpot * gammainc(x/theta, k), "Problem", "dpot");
		opts.lowerBound = @(dd, q) [0 0];
		opts.problemParam = {opts.dpot};
		opts.label = "Gamma model (fixed ddpot)";

		if isempty(opts.startPoint)
			opts.startPoint = @(dd, q) [10 1];
		else
			stp = opts.startPoint;
			opts.startPoint = @(dd, q) stp; 
		end
	end
end

function fdata = fitCurve(q, dd, opts)
	[fobj, gof] = fit(q, dd, opts.fitType, 'lower', opts.lowerBound(dd, q), ...
		'startpoint', opts.startPoint(dd, q), 'problem', opts.problemParam);

	% save fit results
	fdata = struct();
	fdata.obj = fobj;
	fdata.rmse = gof.rmse;
	fdata.r2 = gof.rsquare;
	fdata.mae = mean(abs(dd - fobj(q)));
	fdata.label = opts.label;

	% save parameter values
	parName = opts.parName;

	fdata.parameters = table('Size', [numel(parName), 4], 'VariableTypes', ["string", "double", "double", "double"], ...
					  'VariableNames', ["name", "value", "lower", "upper"]);

	ci = confint(fobj);

	for p = 1:numel(parName)
		fdata.parameters.name(p) = parName(p);
		fdata.parameters.lower(p) = ci(1, p);
		fdata.parameters.upper(p) = ci(2, p);
		fdata.parameters.value(p) = fobj.(parName(p));
	end
end

function plotResult(data, model)
	figure();
	hold on;
	grid on;
	xlabel('q [mm/d]');
	ylabel('dd [km-1]');

	plot(data.q, data.dd, 'o', 'MarkerFaceColor', [1 1 1], 'MarkerEdgeColor', [0 0 0], 'DisplayName', "Empirical data");
	
	qq = linspace(0, 1.1 * max(data.q));
	plot(qq, model.obj(qq), '-', 'DisplayName', model.label);

end
